// generated by verigen.pl on Thu Nov 29 12:58:04 2018
// input file: simple_mem2.cfg

module SIMPLE_MEM2(
	clk, res_n, set, get, din, // inputs
	dout // outputs
 );

input wire clk;
input wire res_n;
input wire set;
input wire get;
input wire [7:0] din;

output reg [7:0] dout;


  //
 // parameter constants 
//---------------------------
parameter ZERO = {8{1'b0}};


  //
 // state encoding constants 
//---------------------------
parameter [1:0] ID	 = 2'b00;
parameter [1:0] SET	 = 2'b01;
parameter [1:0] GET	 = 2'b11;

reg [1:0] STATE, NEXT_STATE;


  //
 // memory instantiations 
//---------------------------
wire MEM_RAM_clk;
wire MEM_RAM_res;
reg MEM_RAM_wr;
reg [1:0] MEM_RAM_wr_addr;
reg [7:0] MEM_RAM_wr_data;
reg MEM_RAM_rd;
reg [1:0] MEM_RAM_rd_addr;
reg MEM_RAM_status;
wire [7:0] MEM_RAM_rd_data;
assign MEM_RAM_clk = clk;
assign MEM_RAM_res_n = res_n;
RAM #(8,4) MEM_RAM(.clk(MEM_RAM_clk), .res_n(MEM_RAM_res), 
	.wr(MEM_RAM_wr), .wr_addr(MEM_RAM_wr_addr[1:0]), .wr_data(MEM_RAM_wr_data[7:0]), 
	.rd(MEM_RAM_rd), .rd_addr(MEM_RAM_rd_addr[1:0]), .rd_data(MEM_RAM_rd_data[7:0]) );


  //
 // constant expressions 
//---------------------------
wire IS_GET; assign IS_GET = (  get & ~set );
wire IS_IDLE; assign IS_IDLE = ( ~get & ~set ) | ( get & set );
wire IS_SET; assign IS_SET = ( ~get &  set );


  //
 // initials for simulator
//---------------------------
initial begin
	STATE = ID;
	MEM_RAM_wr <= 1'b0;
	MEM_RAM_wr_addr <= {1{1'b0}};
	MEM_RAM_wr_data <= {7{1'b0}};
	MEM_RAM_rd <= 1'b0;
	MEM_RAM_rd_addr <= {1{1'b0}};
	MEM_RAM_status <= 1'b0;
end


  //
 // state transfer
//---------------------------
always @( posedge clk or negedge res_n) begin
	if( ~res_n ) begin
		STATE <= ID;
	end else begin
		STATE <= NEXT_STATE;
	end // if
end // always@

always @( * ) begin
		case ( STATE )
		ID : begin
			if ( IS_IDLE ) begin 
				NEXT_STATE <= ID;
				// MEMORY action ...
				MEM_RAM_status <= NOP_MEM_RAM;
				// MEALY output action ...
				dout=ZERO;
			end
			if ( IS_SET ) begin 
				NEXT_STATE <= SET;
				// MEMORY action ...
				MEM_RAM_status <= PUSH_MEM_RAM(din[1:0],din[7:0]);
				// MEALY output action ...
				dout=ZERO;
			end
			if ( IS_GET & (POP_MEM_RAM(din[1:0])==din[7:0]) ) begin 
				NEXT_STATE <= GET;
				// MEMORY action ...
				MEM_RAM_status <= NOP_MEM_RAM;
				// MEALY output action ...
				dout=MEM_RAM_rd_data;
			end
			if ( IS_GET & ~(POP_MEM_RAM(din[1:0])==din[7:0]) ) begin 
				NEXT_STATE <= ID;
				// MEMORY action ...
				MEM_RAM_status <= NOP_MEM_RAM;
				// MEALY output action ...
				dout=ZERO;
			end
		end // ID
		SET : begin
			if ( IS_SET ) begin 
				NEXT_STATE <= SET;
				// MEMORY action ...
				MEM_RAM_status <= PUSH_MEM_RAM(din[1:0],din[7:0]);
				// MEALY output action ...
				dout=ZERO;
			end
			if ( IS_GET ) begin 
				NEXT_STATE <= GET;
				// MEMORY action ...
				MEM_RAM_status <= NOP_MEM_RAM;
				// MEALY output action ...
				dout=MEM_RAM_rd_data;
			end
		end // SET
		GET : begin
			if ( IS_IDLE ) begin 
				NEXT_STATE <= ID;
				// MEMORY action ...
				MEM_RAM_status <= NOP_MEM_RAM;
				// MEALY output action ...
				dout=ZERO;
			end
			if ( IS_SET ) begin 
				NEXT_STATE <= SET;
				// MEMORY action ...
				MEM_RAM_status <= PUSH_MEM_RAM(din[1:0],din[7:0]);
				// MEALY output action ...
				dout=ZERO;
			end
			if ( IS_GET ) begin 
				NEXT_STATE <= GET;
				// MEMORY action ...
				MEM_RAM_status <= NOP_MEM_RAM;
				// MEALY output action ...
				dout=MEM_RAM_rd_data;
			end
			if ( IS_IDLE ) begin 
				NEXT_STATE <= ID;
				// MEMORY action ...
				MEM_RAM_status <= NOP_MEM_RAM;
				// MEALY output action ...
				dout=ZERO;
			end
		end // GET
		default begin 
			NEXT_STATE <= ID;
			dout[7:0] <= {8{1'b0}};
		end
		endcase
end // always@


  //
 // memory functions
//---------------------------
function [7:0] POP_MEM_RAM;
   input [1:0] rd_addr;
begin
   MEM_RAM_rd = 1'b1;
   MEM_RAM_rd_addr = rd_addr;
   POP_MEM_RAM = MEM_RAM_rd_data;
end
endfunction

function PUSH_MEM_RAM;
   input [1:0] wr_addr;
   input [7:0] wr_data;
begin
   MEM_RAM_wr = 1'b1;
   MEM_RAM_wr_addr = wr_addr;
   MEM_RAM_wr_data = wr_data;
   PUSH_MEM_RAM = 1'b1;
end
endfunction

function TOP_MEM_RAM;
   input [1:0] rd_addr;
begin
   MEM_RAM_rd = 1'b1;
   MEM_RAM_rd_addr = rd_addr;
   TOP_MEM_RAM = MEM_RAM_rd_data;
end
endfunction

function NOP_MEM_RAM;
begin
   //MEM_RAM_rd = 1'b0;
   MEM_RAM_wr = 1'b0;
   MEM_RAM_wr_addr = {2{1'b0}};
   MEM_RAM_wr_data = {8{1'b0}};
   NOP_MEM_RAM = 1'b1;
endfunction

endmodule


  //
 // memory
//---------------------------
module RAM ( clk, res_n, wr, wr_addr, wr_data, rd, rd_addr, rd_data );

parameter data_width = 32;
parameter entries = 16;
parameter addr_width = $clog2(entries);

input  wire clk, res_n, rd, wr;
input  wire [data_width-1:0] wr_data;
input  wire [addr_width-1:0] rd_addr, wr_addr;
output wire [data_width-1:0] rd_data;

parameter ZERO_ADDR = {addr_width{1'b0}};
parameter ZERO_DATA = {data_width{1'b0}};
parameter ONE_DATA = {data_width{1'b1}};
    
reg    [data_width-1:0] MEMORY [entries-1:0];

integer i;

initial begin
   // Clear RAM before simulation start
   for ( i = 0; i < entries; i = i + 1 ) begin
       MEMORY[ i ] <= ZERO_DATA;
   end
end

// IN-CYCLE READ ACCESS
assign rd_data = {data_width{rd & res_n}} & MEMORY[rd_addr];

always @ ( posedge clk ) begin
    // WRITE ACCESS
   MEMORY[wr_addr] <= {data_width{wr & res_n}} & wr_data;
end

endmodule
